import * as _codemirror_state from '@codemirror/state';
import { StateEffectType, StateField } from '@codemirror/state';
import { Parinfer, ParinferError } from 'parinfer';
import { EditorView } from '@codemirror/view';

declare global {
    interface Window {
        parinfer?: Parinfer;
    }
}

/**
 * Effect to set the enabled state.
 */
declare const setEnabledEffect: StateEffectType<boolean>;
/**
 * Type representing a Parinfer mode.
 */
type ParinferMode = "smart" | "indent" | "paren";
/**
 * Effect to switch the Parinfer mode.
 */
declare const switchModeEffect: StateEffectType<ParinferMode>;
interface InvertibleFieldValue<T> {
    id: number;
    invertsId?: number;
    previous: T | null;
    current: T | null;
}
/**
 * Switches the Parinfer mode for the provided editor view.
 * @param view the editor view
 * @param mode the Parinfer mode to switch to
 */
declare function switchMode(view: EditorView, mode: ParinferMode): void;
/**
 * Disables Parinfer for the provided editor view.
 * @param view the editor view
 */
declare function disableParinfer(view: EditorView): void;
/**
 * Enables Parinfer for the provided editor view.
 * @param view the editor view
 */
declare function enableParinfer(view: EditorView): void;
/**
 * @returns the CodeMirror6 Parinfer extension in the form of an array of extensions
 */
declare function parinferExtension(): (StateField<boolean> | StateField<ParinferMode> | StateField<InvertibleFieldValue<ParinferError>> | _codemirror_state.Extension)[];

export { type ParinferMode, disableParinfer, enableParinfer, parinferExtension, setEnabledEffect, switchMode, switchModeEffect };
